//libraries
#include <DHT.h>
#include<Wire.h> 
#include <BH1750.h>

//-----------------pin definitions and variables related to temperature-----------------------
#define DHT1_PIN 7     // what pin we're connected to
#define DHTTYPE DHT22   // DHT 22  (AM2302)
#define AIR_COOLER_PIN 6 //air cooler output

int tempSetPoint;
float tempRead, deltaSetTemp, pwmValueAirCooler,
      tempVeryCold, tempCold, tempNormal, tempHot, tempVeryHot, //fuzzy member for delta temperature
      tempRule1, tempRule2a, tempRule2b, tempRule3a, tempRule3b, tempRule4a, tempRule4b, tempRule5, //fuzzy rules for air cooler setting
      outputAirCooler; //output rules
String outputFuzzyAirCooler;

//-----------------pin definitions and variables related to humidity--------------------------
#define HUMIDIFIER_PIN 8

int humiditySetPoint;
float humidityRead, deltaSetHumidity, pwmValueHumidifier,
      humidityVeryWet, humidityWet, humidityNormal, humidityDry, humidityVeryDry, //fuzzy member for delta humidity
      humidityRule1, humidityRule2a, humidityRule2b, humidityRule3a, humidityRule3b, humidityRule4a, humidityRule4b, humidityRule5, //fuzzy rules for humiditifier setting
      outputHumidifier; //output rules
String outputFuzzyHumidifier;

//-----------------pin definitions and variables related to light-----------------------------
#define LIGHT_PIN 5

int lightSetPoint;
float lightRead, deltaSetLight, pwmValueLeds,
      lightVeryLarge, lightLarge, lightNormal, lightSmall, lightVerySmall, //fuzzy member for delta light
      lightRule1, lightRule2a, lightRule2b, lightRule3a, lightRule3b, lightRule4a, lightRule4b, lightRule5, //fuzzy rules for light setting
      outputLeds; //output rules
String outputFuzzyLeds;

//-----------------pin definitions and variables related to air-------------------------------
#define MQ_PIN A0
#define FAN_PIN 3

int airQualitySetPoint;
float airQualityRead, deltaSetAirQ, pwmValueFan,
      airQualityVeryGood, airQualityGood, airQualityAverage, airQualityBad, airQualityVeryBad, //fuzzy member for read and set air quality difference
      airQualityRule1, airQualityRule2a, airQualityRule2b, airQualityRule3a, airQualityRule3b, airQualityRule4a, airQualityRule4b, airQualityRule5, //fuzzy rules for fan setting
      outputFan; //output fan rule
String outputFuzzyFan;

//Intializations
DHT dht1(DHT1_PIN, DHTTYPE); // Initialize DHT sensor for normal 16mhz Arduino
BH1750 GY30;

void setup() {
  Serial.begin(9600);
  dht1.begin();
  //Wire.begin(); 
  //GY30.begin();

  pinMode(AIR_COOLER_PIN, OUTPUT);
  pinMode(HUMIDIFIER_PIN, OUTPUT);
  pinMode(LIGHT_PIN, OUTPUT);
  pinMode(FAN_PIN, OUTPUT);

  digitalWrite(LIGHT_PIN, LOW); //initial light
  digitalWrite(FAN_PIN, LOW); //intial fan
}

void loop() {
  //temperature functions
  //temp_set(); //set required temperature for strawberry
  //temp_readSensor(); //function for reading temperature from sensor
  //temp_fuzzification();
  //temp_fuzzy_rule();
  //temp_defuzzification();
  //temp_output();
  //temp_fuzzy_output();

  //humidity functions
  //humidity_set(); //set required humidity for strawberry
  //humidity_readSensor(); //function for reading humidity from sensor
  //humidity_fuzzification();
  //humidity_fuzzy_rule();
  //humidity_defuzzification();
  //humidity_output();
  //humidity_fuzzy_output();

  //light functions
  light_set(); //set required light intensity for strawberry
  light_readSensor(); //function for reading light intensity from sensor
  light_fuzzification();
  light_fuzzy_rule();
  light_defuzzification();
  light_output();
  light_fuzzy_output();

  //air funtions
  //air_quality_set();
  //air_quality_readSensor();
  //air_quality_fuzzification();
  //air_quality_fuzzy_rule();
  //air_quality_defuzzification();
  //air_quality_output();
  //air_quality_fuzzy_output();
    
  Serial.println();
  delay(10000);
}

//--------------temperature-------------------
void temp_set(){
  tempSetPoint = 20; //set required temperature set point for strawberry
}

void temp_readSensor(){
  //tempRead = dht1.readTemperature(); //read temperature from temperature sensor
  tempRead = 20;

  if(tempRead > tempSetPoint){
    deltaSetTemp = abs(tempRead-tempSetPoint); //find the temperature differece between sensor input and the set temperature
  }else{
    deltaSetTemp = 0;
  }
      
  Serial.print("Temperature: ");
  Serial.println(tempRead);
  Serial.print("Difference: ");
  Serial.println(deltaSetTemp);
}

void temp_fuzzification(){
  deltaTempVeryCold();
  deltaTempCold();
  deltaTempNormal();
  deltaTempHot();
  deltaTempVeryHot();

 /* pwmVoltageVeryLow();
  pwmVoltageLow();
  pwmVoltageNormal();
  pwmVoltageHigh();
  pwmVoltageVeryHigh();*/
}

unsigned char deltaTempVeryCold(){
  if(deltaSetTemp <= 1){
    tempVeryCold = 1;
  }else if(deltaSetTemp >= 1 && deltaSetTemp <= 2){
    tempVeryCold = (2 - deltaSetTemp)/1;
  }else if(deltaSetTemp >= 2){
    tempVeryCold = 0;
  }
  Serial.print("Delta Very Cold: ");
  Serial.println(tempVeryCold);
  return tempVeryCold;
}

unsigned char deltaTempCold(){
  if(deltaSetTemp <= 1){
    tempCold = 0;
  }else if(deltaSetTemp >= 1 && deltaSetTemp <= 2){
    tempCold = (deltaSetTemp - 1)/1;
  }else if(deltaSetTemp >= 2 && deltaSetTemp <= 3){
    tempCold = (3 - deltaSetTemp)/1;
  }else if(deltaSetTemp >= 3){
    tempCold = 0;
  }
  Serial.print("Delta Cold: ");
  Serial.println(tempCold);
  return tempCold;
}

unsigned char deltaTempNormal(){
  if(deltaSetTemp <= 2){
    tempNormal = 0;
  }else if(deltaSetTemp >= 2 && deltaSetTemp <= 3){
    tempNormal = (deltaSetTemp - 2)/1;
  }else if(deltaSetTemp >=3 && deltaSetTemp <= 4){
    tempNormal = (4 - deltaSetTemp)/1;
  }else if(deltaSetTemp >= 4){
    tempNormal = 0;
  }
  Serial.print("Delta Normal: ");
  Serial.println(tempNormal);
  return tempNormal;
}

unsigned char deltaTempHot(){
  if(deltaSetTemp <= 3){
    tempHot = 0;
  }else if(deltaSetTemp >= 3 && deltaSetTemp <= 4){
    tempHot = (deltaSetTemp - 3)/1;
  }else if(deltaSetTemp >=4 && deltaSetTemp <= 5){
    tempHot = (5 - deltaSetTemp)/1;
  }else if(deltaSetTemp >= 5){
    tempHot = 0;
  }
  Serial.print("Delta Hot: ");
  Serial.println(tempHot);
  return tempHot;
}

unsigned char deltaTempVeryHot(){
  if(deltaSetTemp <= 4){
    tempVeryHot = 0;
  }else if(deltaSetTemp >=4 && deltaSetTemp <= 5){
    tempVeryHot = (deltaSetTemp - 4)/1;
  }else if(deltaSetTemp >= 5){
    tempVeryHot = 1;
  }
  Serial.print("Delta Very Hot: ");
  Serial.println(tempVeryHot);
  return tempVeryHot;
}

void temp_fuzzy_rule(){
  tempRule1  = 40 - (tempVeryCold * 20);
  tempRule2a = 20 + (tempCold * 20);
  tempRule2b = 60 - (tempCold * 20);
  tempRule3a = 40 + (tempNormal * 20); 
  tempRule3b = 80 - (tempNormal * 20);
  tempRule4a = 60 + (tempHot * 20);
  tempRule4b = 100 - (tempHot * 20);
  tempRule5 = 80 + (tempVeryHot * 20);
}

void temp_defuzzification(){
  outputAirCooler = ((tempRule1 * tempVeryCold) + (tempRule2a * tempCold) + (tempRule2b * tempCold) + (tempRule3a * tempNormal) + (tempRule3b * tempNormal) + (tempRule4a * tempHot) + (tempRule4b * tempHot) + (tempRule5 * tempVeryHot)) / (tempVeryCold + tempCold + tempCold + tempNormal + tempNormal + tempHot + tempHot + tempVeryHot);
  Serial.print("Air Cooler Output Duty: ");
  Serial.println(outputAirCooler);
}

void temp_output(){
  pwmValueAirCooler = map(outputAirCooler, 0, 100, 0, 255);
  Serial.print("Air Cooler Output PWM: ");
  Serial.println(pwmValueAirCooler);
  analogWrite(AIR_COOLER_PIN, pwmValueAirCooler);
}

void temp_fuzzy_output(){
  if(outputAirCooler <= 20){
    outputFuzzyAirCooler = "Very Low";
  }else if(outputAirCooler >= 20 && outputAirCooler <= 40){
    outputFuzzyAirCooler = "Low";
  }else if(outputAirCooler >= 40 && outputAirCooler <= 60){
    outputFuzzyAirCooler = "Normal";
  }else if(outputAirCooler >= 60 && outputAirCooler <= 80){
    outputFuzzyAirCooler = "High";
  } else if (outputAirCooler >= 80 && outputAirCooler <= 100){
    outputFuzzyAirCooler = "Very High";
  }
  Serial.print("Fuzzy Output Air Cooler: ");
  Serial.println(outputFuzzyAirCooler);
}

//--------------humidity-------------------
void humidity_set(){
  humiditySetPoint = 90; //set required humidity set point for strawberry
}

void humidity_readSensor(){
  //humidityRead = dht1.readHumidity(); //read humidity sensor input
  humidityRead = 65; 

  if(humidityRead < humiditySetPoint){
    deltaSetHumidity = abs(humiditySetPoint-humidityRead); //find the humidity differece between sensor input and the set humidity
  }else{
    deltaSetHumidity = 0;
  }

  Serial.print("Humidity: ");
  Serial.println(humidityRead);
  Serial.print("Difference: ");
  Serial.println(deltaSetHumidity);
}

void humidity_fuzzification(){
  deltaHumidityVeryWet();
  deltaHumidityWet();
  deltaHumidityNormal();
  deltaHumidityDry();
  deltaHumidityVeryDry();
}

unsigned char deltaHumidityVeryWet(){
  if(deltaSetHumidity <= 20){
    humidityVeryWet = 1;
  }else if(deltaSetHumidity >= 20 && deltaSetHumidity <= 40){
    humidityVeryWet = (40 - deltaSetHumidity)/20;
  }else if(deltaSetHumidity >= 40){
    humidityVeryWet = 0;
  }
  Serial.print("Delta Very Wet: ");
  Serial.println(humidityVeryWet);
  return humidityVeryWet;
}

unsigned char deltaHumidityWet(){
  if(deltaSetHumidity <= 20){
    humidityWet = 0;
  }else if(deltaSetHumidity >= 20 && deltaSetHumidity <= 40){
    humidityWet = (deltaSetHumidity - 20)/20;
  }else if(deltaSetHumidity >= 40 && deltaSetHumidity <= 60){
    humidityWet = (60 - deltaSetHumidity)/20;
  }else if(deltaSetHumidity >= 60){
    humidityWet = 0;
  }
  Serial.print("Delta Wet: ");
  Serial.println(humidityWet);
  return humidityWet;
}

unsigned char deltaHumidityNormal(){
  if(deltaSetHumidity <= 40){
    humidityNormal = 0;
  }else if(deltaSetHumidity >= 40 && deltaSetHumidity <= 60){
    humidityNormal = (deltaSetHumidity - 40)/20;
  }else if(deltaSetHumidity >= 60 && deltaSetHumidity <= 80){
    humidityNormal = (80 - deltaSetHumidity)/20;
  }else if(deltaSetHumidity >= 80){
    humidityNormal = 0;
  }
  Serial.print("Delta Normal: ");
  Serial.println(humidityNormal);
  return humidityNormal;
}

unsigned char deltaHumidityDry(){
  if(deltaSetHumidity <= 60){
    humidityDry = 0;
  }else if(deltaSetHumidity >= 60 && deltaSetHumidity <= 80){
    humidityDry = (deltaSetHumidity - 60)/20;
  }else if(deltaSetHumidity >= 80 && deltaSetHumidity <= 100){
    humidityDry = (100 - deltaSetHumidity)/20;
  }else if(deltaSetHumidity >= 100){
    humidityDry = 0;
  }
  Serial.print("Delta Dry: ");
  Serial.println(humidityDry);
  return humidityDry;
}

unsigned char deltaHumidityVeryDry(){
  if(deltaSetHumidity <= 80){
    humidityVeryDry = 0;
  }else if(deltaSetHumidity >= 80 && deltaSetHumidity <= 100){
    humidityVeryDry = (deltaSetHumidity - 80)/20;
  }else if(deltaSetHumidity >= 100){
    humidityVeryDry = 1;
  }
  Serial.print("Delta Very Dry: ");
  Serial.println(humidityVeryDry);
  return humidityVeryDry;
}

void humidity_fuzzy_rule(){
  humidityRule1  = 80 + (humidityVeryDry * 20);
  humidityRule2a = 60 + (humidityDry * 20);
  humidityRule2b = 100 - (humidityDry * 20);
  humidityRule3a = 40 + (humidityNormal * 20); 
  humidityRule3b = 80 - (humidityNormal * 20);
  humidityRule4a = 20 + (humidityWet * 20);
  humidityRule4b = 60 - (humidityWet * 20);
  humidityRule5 = 40 - (humidityVeryWet * 20);
}

void humidity_defuzzification(){
  outputHumidifier = ((humidityRule1 * humidityVeryDry) + (humidityRule2a * humidityDry) + (humidityRule2b * humidityDry) + (humidityRule3a * humidityNormal) + (humidityRule3b * humidityNormal) + (humidityRule4a * humidityWet) + (humidityRule4b * humidityWet) + (humidityRule5 * humidityVeryWet)) / (humidityVeryDry + humidityDry + humidityDry + humidityNormal + humidityNormal + humidityWet + humidityWet + humidityVeryWet);
  Serial.print("Humidifier Output Duty: ");
  Serial.println(outputHumidifier);
}

void humidity_output(){
  pwmValueHumidifier = map(outputHumidifier, 0, 100, 0, 255);
  Serial.print("Humidifier Output PWM: ");
  Serial.println(pwmValueHumidifier);
  analogWrite(HUMIDIFIER_PIN, pwmValueHumidifier);
}

void humidity_fuzzy_output(){
  if(outputHumidifier <= 20){
    outputFuzzyHumidifier = "Very Slow";
  }else if(outputHumidifier >= 20 && outputHumidifier <= 40){
    outputFuzzyHumidifier = "Slow";
  }else if(outputHumidifier >= 40 && outputHumidifier <= 60){
    outputFuzzyHumidifier = "Normal";
  }else if(outputHumidifier >= 60 && outputHumidifier <= 80){
    outputFuzzyHumidifier = "Fast";
  } else if (outputHumidifier >= 80 && outputHumidifier <= 100){
    outputFuzzyHumidifier = "Very Fast";
  }
  Serial.print("Fuzzy Output Humidifier: ");
  Serial.println(outputFuzzyHumidifier);
}

//-------------light-----------------------
void light_set(){
  lightSetPoint = 2254; //set required light set point for strawberry
}

void light_readSensor(){
  //lightRead = GY30.readLightLevel(); //read light intensity from light sensor
  lightRead = 754;

  if(lightRead < lightSetPoint){
    deltaSetLight = abs(lightSetPoint-lightRead); //find the light differece between sensor input and the set light intensity
  }else{
    deltaSetLight = 0;
  }

  Serial.print("Light Intensity: ");
  Serial.println(lightRead);
  Serial.print("Difference: ");
  Serial.println(deltaSetLight);  
}

void light_fuzzification(){
  deltaLightVerySmall();
  deltaLightSmall();
  deltaLightNormal();
  deltaLightLarge();
  deltaLightVeryLarge();
}

unsigned char deltaLightVerySmall(){
  if(deltaSetLight <= 400){
    lightVerySmall = 1;
  }else if(deltaSetLight >= 400 && deltaSetLight <= 800){
    lightVerySmall = (800 - deltaSetLight)/400;
  }else if(deltaSetLight >= 800){
    lightVerySmall = 0;
  }
  Serial.print("Delta Very Small: ");
  Serial.println(lightVerySmall);
  return lightVerySmall;
}

unsigned char deltaLightSmall(){
  if(deltaSetLight <= 400){
    lightSmall = 0;
  }else if(deltaSetLight >= 400 && deltaSetLight <= 800){
    lightSmall = (deltaSetLight - 400)/400;
  }else if(deltaSetLight >= 800 && deltaSetLight <= 1200){
    lightSmall = (1200 - deltaSetLight)/400;
  }else if(deltaSetLight >= 1200){
    lightSmall = 0;
  }
  Serial.print("Delta Small: ");
  Serial.println(lightSmall);
  return lightSmall;
}

unsigned char deltaLightNormal(){
  if(deltaSetLight <= 800){
    lightNormal = 0;
  }else if(deltaSetLight >= 800 && deltaSetLight <= 1200){
    lightNormal = (deltaSetLight - 800)/400;
  }else if(deltaSetLight >= 1200 && deltaSetLight <= 1600){
    lightNormal = (1600 - deltaSetLight)/400;
  }else if(deltaSetLight >= 1600){
    lightNormal = 0;
  }
  Serial.print("Delta Normal: ");
  Serial.println(lightNormal);
  return lightNormal;
}

unsigned char deltaLightLarge(){
  if(deltaSetLight <= 1200){
    lightLarge = 0;
  }else if(deltaSetLight >= 1200 && deltaSetLight <= 1600){
    lightLarge = (deltaSetLight - 1200)/400;
  }else if(deltaSetLight >= 1600 && deltaSetLight <= 2000){
    lightLarge = (2000 - deltaSetLight)/400;
  }else if(deltaSetLight >= 2000){
    lightLarge = 0;
  }
  Serial.print("Delta Large: ");
  Serial.println(lightLarge);
  return lightLarge;
}

unsigned char deltaLightVeryLarge(){
  if(deltaSetLight <= 1600){
    lightVeryLarge = 0;
  }else if(deltaSetLight >= 1600 && deltaSetLight <= 2000){
    lightVeryLarge = (deltaSetLight - 1600)/400;
  }else if(deltaSetLight >= 2000){
    lightVeryLarge = 1;
  }
  Serial.print("Delta Very Large: ");
  Serial.println(lightVeryLarge);
  return lightVeryLarge;
}

void light_fuzzy_rule(){
  lightRule1  = 80 + (lightVeryLarge * 20);
  lightRule2a = 60 + (lightLarge * 20);
  lightRule2b = 100 - (lightLarge * 20);
  lightRule3a = 40 + (lightNormal * 20); 
  lightRule3b = 80 - (lightNormal * 20);
  lightRule4a = 20 + (lightSmall * 20);
  lightRule4b = 60 - (lightSmall * 20);
  lightRule5 = 40 - (lightVerySmall * 20);
}

void light_defuzzification(){
  outputLeds = ((lightRule1 * lightVeryLarge) + (lightRule2a * lightLarge) + (lightRule2b * lightLarge) + (lightRule3a * lightNormal) + (lightRule3b * lightNormal) + (lightRule4a * lightSmall) + (lightRule4b * lightSmall) + (lightRule5 * lightVerySmall)) / (lightVeryLarge + lightLarge + lightLarge + lightNormal + lightNormal + lightSmall + lightSmall + lightVerySmall);
  Serial.print("Leds Output Duty: ");
  Serial.println(outputLeds);
}

void light_output(){
  pwmValueLeds = map(outputLeds, 0, 100, 0, 255);
  Serial.print("Leds Output PWM: ");
  Serial.println(pwmValueLeds);
  analogWrite(LIGHT_PIN, pwmValueLeds);
}

void light_fuzzy_output(){
  if(outputLeds <= 20){
    outputFuzzyLeds = "Very Low";
  }else if(outputLeds >= 20 && outputLeds <= 40){
    outputFuzzyLeds = "Low";
  }else if(outputLeds >= 40 && outputLeds <= 60){
    outputFuzzyLeds = "Normal";
  }else if(outputLeds >= 60 && outputLeds <= 80){
    outputFuzzyLeds = "High";
  } else if (outputLeds >= 80 && outputLeds <= 100){
    outputFuzzyLeds = "Very High";
  }
  Serial.print("Fuzzy Output Leds: ");
  Serial.println(outputFuzzyLeds);
}

/*void light_control(){
  analogWrite(LIGHT_PIN, 255);
}*/

//------------------air quality--------------------
void air_quality_set(){
  airQualitySetPoint = 20; //set required air quality set point for strawberry
}

void air_quality_readSensor(){
  //airQualityRead = analogRead(MQ_PIN); //read air quality from MQT-135 sensor
  airQualityRead = 48;

  if(airQualityRead > airQualitySetPoint){
    deltaSetAirQ = abs(airQualityRead-airQualitySetPoint); //find the air quality differece between sensor input and the set air quality
  }else{
    deltaSetAirQ = 0;
  }
  
  Serial.print("Air Quality: ");
  Serial.println(airQualityRead);
  Serial.print("Difference: ");
  Serial.println(deltaSetAirQ);
}  

void air_quality_fuzzification(){
  deltaAirQualityVeryGood();
  deltaAirQualityGood();
  deltaAirQualityAverage();
  deltaAirQualityBad();
  deltaAirQualityVeryBad();
}

unsigned char deltaAirQualityVeryGood(){
  if(deltaSetAirQ <= 25){
    airQualityVeryGood = 1;
  }else if(deltaSetAirQ >= 25 && deltaSetAirQ <= 75){
    airQualityVeryGood = (75 - deltaSetAirQ)/50;
  }else if(deltaSetAirQ >= 75){
    airQualityVeryGood = 0;
  }
  Serial.print("Delta Very Good: ");
  Serial.println(airQualityVeryGood);
  return airQualityVeryGood;
}

unsigned char deltaAirQualityGood(){
  if(deltaSetAirQ <= 25){
    airQualityGood = 0;
  }else if(deltaSetAirQ >= 25 && deltaSetAirQ <= 75){
    airQualityGood = (deltaSetAirQ - 25)/50;
  }else if(deltaSetAirQ >= 75 && deltaSetAirQ <= 125){
    airQualityGood = (125 - deltaSetAirQ)/50;
  }else if(deltaSetAirQ >= 125){
    airQualityGood = 0;
  }
  Serial.print("Delta Good: ");
  Serial.println(airQualityGood);
  return airQualityGood;
}

unsigned char deltaAirQualityAverage(){
  if(deltaSetAirQ <= 75){
    airQualityAverage = 0;
  }else if(deltaSetAirQ >= 75 && deltaSetAirQ <= 125){
    airQualityAverage = (deltaSetAirQ - 75)/50;
  }else if(deltaSetAirQ >= 125 && deltaSetAirQ <= 175){
    airQualityAverage = (175 - deltaSetAirQ)/50;
  }else if(deltaSetAirQ >= 175){
    airQualityAverage = 0;
  }
  Serial.print("Delta Average: ");
  Serial.println(airQualityAverage);
  return airQualityAverage;
}

unsigned char deltaAirQualityBad(){
  if(deltaSetAirQ <= 125){
    airQualityBad = 0;
  }else if(deltaSetAirQ >= 125 && deltaSetAirQ <= 175){
    airQualityBad = (deltaSetAirQ - 125)/50;
  }else if(deltaSetAirQ >= 175 && deltaSetAirQ <= 225){
    airQualityBad = (225 - deltaSetAirQ)/50;
  }else if(deltaSetAirQ >= 225){
    airQualityBad = 0;
  }
  Serial.print("Delta Bad: ");
  Serial.println(airQualityBad);
  return airQualityBad;
}

unsigned char deltaAirQualityVeryBad(){
  if(deltaSetAirQ <= 175){
    airQualityVeryBad = 0;
  }else if(deltaSetAirQ >= 175 && deltaSetAirQ <= 225){
    airQualityVeryBad = (deltaSetAirQ - 175)/50;
  }else if(deltaSetAirQ >= 225){
    airQualityVeryBad = 1;
  }
  Serial.print("Delta Very Bad: ");
  Serial.println(airQualityVeryBad);
  return airQualityVeryBad;
}

void air_quality_fuzzy_rule(){
  airQualityRule1  = 80 + (airQualityVeryBad * 20);
  airQualityRule2a = 60 + (airQualityBad * 20);
  airQualityRule2b = 100 - (airQualityBad * 20);
  airQualityRule3a = 40 + (airQualityAverage * 20); 
  airQualityRule3b = 80 - (airQualityAverage * 20);
  airQualityRule4a = 20 + (airQualityGood * 20);
  airQualityRule4b = 60 - (airQualityGood * 20);
  airQualityRule5 = 40 - (airQualityVeryGood * 20);
}

void air_quality_defuzzification(){
  outputFan = ((airQualityRule1 * airQualityVeryBad) + (airQualityRule2a * airQualityBad) + (airQualityRule2b * airQualityBad) + (airQualityRule3a * airQualityAverage) + (airQualityRule3b * airQualityAverage) + (airQualityRule4a * airQualityGood) + (airQualityRule4b * airQualityGood) + (airQualityRule5 * airQualityVeryGood)) / (airQualityVeryBad + airQualityBad + airQualityBad + airQualityAverage + airQualityAverage + airQualityGood + airQualityGood + airQualityVeryGood);
  Serial.print("Fan Output Duty: ");
  Serial.println(outputFan);
}

void air_quality_output(){
  pwmValueFan = map(outputFan, 0, 100, 0, 255);
  Serial.print("Fan Output PWM: ");
  Serial.println(pwmValueFan);
  analogWrite(FAN_PIN, pwmValueFan);
}

void air_quality_fuzzy_output(){
  if(outputFan <= 20){
    outputFuzzyFan = "Very Slow";
  }else if(outputFan >=20 && outputFan <= 40){
    outputFuzzyFan = "Slow";
  }else if(outputFan >=40 && outputFan <= 60){
    outputFuzzyFan = "Normal";
  }else if(outputFan >=60 && outputFan <= 80){
    outputFuzzyFan = "Fast";
  } else if (outputFan >=80 && outputFan <= 100){
    outputFuzzyFan = "Very Fast";
  }
  Serial.print("Fuzzy Output Fan: ");
  Serial.println(outputFuzzyFan);
}
